/*
 ************************************************************************************************
 *
 *  OSA cooperative RTOS for microcontrollers PIC (Microchip) and 8-bit AVR (Atmel)
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:            http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *                  http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:           osa_system.c
 *
 *  Compilers:      HT-PICC STD
 *                  HT-PICC PRO
 *                  HT-PICC18 STD
 *                  Microchip C18
 *                  Microchip C30
 *                  MikroC
 *                  CCS
 *                  WinAVR
 *                  IAR
 *
 *  Programmer:     Timofeev Victor
 *                  osa@pic24.ru, testerplus@mail.ru
 *
 *  Description:    System functions
 *
 *  History:        14.03.2010 -    File created
 *
 ************************************************************************************************
 */



/*
 ************************************************************************************************
 *                                                                                              *
 *                         C R I T I C A L   S E C T I O N S                                    *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------

/*
 ********************************************************************************
 *                                                                              *
 *   void OS_EnterCriticalSection (void)                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *   description:   Enter critical section.                                     *
 *                  This function disables interrupts (with saving current      *
 *                  state) and sets system flag bInCriticalSection              *
 *                                                                              *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc16.c"                                         *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_mcc30.c"                                          *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(OS_EnterCriticalSection_DEFINED)
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
#ifdef      __OSA18__
//------------------------------------------------------------------------------

    void OS_EnterCriticalSection (void)
    {
        __osa_word temp;

        temp = OS_DI();
        OS_Flags.bInCriticalSection = 1;

        OS_Flags.bGIE_CTemp = 0;
        if (temp & 0x80) OS_Flags.bGIE_CTemp = 1;

        OS_Flags.bGIEL_CTemp = 0;
        if (temp & 0x40) OS_Flags.bGIEL_CTemp = 1;
    }


//------------------------------------------------------------------------------
#else   //  __OSA18__
//------------------------------------------------------------------------------

    void OS_EnterCriticalSection (void)
    {
        OS_Flags.bGIE_CTemp = 0;
        if (OS_DI() & 0x80) OS_Flags.bGIE_CTemp = 1;
        OS_Flags.bInCriticalSection = 1;

    }

//------------------------------------------------------------------------------
#endif  //  __OSA18__
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------




/*
 ********************************************************************************
 *                                                                              *
 *   void OS_LeaveCriticalSection (void)                                        *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    Leave critical section.                                     *
 *                  This function restore interrupt state from OS_Flag temp bits*
 *                  and clears system flag bInCriticalSection                   *
 *                                                                              *
 *  parameters:     none                                                        *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc16.c"                                         *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_mcc30.c"                                          *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(OS_LeaveCriticalSection_DEFINED)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#ifdef      __OSA18__
//------------------------------------------------------------------------------

    void OS_LeaveCriticalSection (void)
    {
        char temp;
        OS_Flags.bInCriticalSection = 0;
        temp = 0;
        if (OS_Flags.bGIE_CTemp)  temp |= 0x80;
        if (OS_Flags.bGIEL_CTemp) temp |= 0x40;
        OS_RI(temp);
    }

//------------------------------------------------------------------------------
#else   //  __OSA18__
//------------------------------------------------------------------------------

    void OS_LeaveCriticalSection (void)
    {
        OS_Flags.bInCriticalSection = 0;
        if (OS_Flags.bGIE_CTemp) OS_RI(0x80);
    }

//------------------------------------------------------------------------------
#endif  //  __OSA18__
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif  //  OS_ENABLE_CRITICAL_SECTION
//------------------------------------------------------------------------------








/*
 ************************************************************************************************
 *                                                                                              *
 *                                   D E L A Y S   I N   T A S K                                *
 *                                                                                              *
 ************************************************************************************************
 */

//------------------------------------------------------------------------------
#ifdef OS_ENABLE_TTIMERS
//------------------------------------------------------------------------------

/*
 ********************************************************************************
 *                                                                              *
 *   void _OS_InitDelay (_OST_TTIMER Delay)                                     *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel)                 *
 *                                                                              *
 *                  Init task timer delay.Set timer variable in task descriptor,*
 *                  sets bit bDelay and clears bit bCanContinue.                *
 *                                                                              *
 *                  _OS_CurTask must point to descriptor.                       *
 *                                                                              *
 *                                                                              *
 *  parameters:     Delay   - time of delay in system ticks                     *
 *                                                                              *
 *  on return:      none                                                        *
 *                                                                              *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                  "port\osa_picc16.c"                                         *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_picc18.c"                                         *
 *                  "port\osa_mcc18.c"                                          *
 *                  "port\osa_mcc30.c"                                          *
 *                                                                              *
 ********************************************************************************
 */




//------------------------------------------------------------------------------
#if !defined(_OS_InitDelay_DEFINED)
//------------------------------------------------------------------------------

    void _OS_InitDelay (_OST_TTIMER Delay)
    {

        OS_TASK_ATOMIC_WRITE_A(
            _OS_bTaskCanContinue = 0;
            _OS_CurTask->State.bDelay = 0;
        );

        if (Delay)
        {
            Delay ^= (_OST_TTIMER)-1;
            Delay ++;
            _OS_CurTask->Timer = Delay;
            OS_TASK_ATOMIC_WRITE_A(
                _OS_CurTask->State.bReady = 1;
                _OS_CurTask->State.bDelay = 1;
            );
            #if defined(_OS_CUR_FLAGS_IN_OS_state)  /*100312.1*/
                _OS_bTaskDelay = 1;
                _OS_bTaskReady = 1;
            #endif
        }
    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_InitDelay_DEFINED)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
#endif  // #ifdef OS_TIMER
//------------------------------------------------------------------------------




//-----------------------------------------------------------------
#ifdef OS_ENABLE_OS_TIMER
#ifndef OS_USE_INLINE_TIMER
//-----------------------------------------------------------------
void OS_Timer (void)
{
    __OS_TimerInline();
}
//-----------------------------------------------------------------
#endif  // OS_ENABLE_OS_TIMER
#endif  // OS_USE_INLINE_TIMER
//-----------------------------------------------------------------






/*
 ********************************************************************************
 *                                                                              *
 *  bit _OS_CheckEvent (char bEvent)                                            *
 *                                                                              *
 *------------------------------------------------------------------------------*
 *                                                                              *
 *  description:    (Internal function called by system kernel througth         *
 *                  waiting services)                                           *
 *                                                                              *
 *                  Check condition of bEvent. Accordint to bEvent value and    *
 *                  current state of bReady, bDelay and bCanContinue flags,     *
 *                  task becames ready to execute or still remains in waiting   *
 *                  mode.                                                       *
 *                                                                              *
 *  parameters:     bEvent - zero, or non-zero condition                        *
 *                                                                              *
 *  on return:      1 - when condition is true, or timeout occured              *
 *                  0 - condition is false and no timeout                       *
 *                                                                              *
 *  Overloaded in:  "port\osa_picc12.c"                                         *
 *                  "port\osa_picc16.c"                                         *
 *                  "port\osa_picc18.c"                                         *
 *                  "port\osa_mikroc16.c"                                       *
 *                  "port\osa_mikroc18.c"                                       *
 *                  "port\osa_mcc18.c"                                          *
 *                  "port\osa_mcc30.c"                                          *
 *                  "port\osa_ccs16.c"                                          *
 *                  "port\osa_ccs18.c"                                          *
 *                                                                              *
 ********************************************************************************
 */

//------------------------------------------------------------------------------
#if !defined(_OS_CheckEvent_DEFINED)
//------------------------------------------------------------------------------

    __osa_bool _OS_CheckEvent (char bEvent)      // *91029*
    {
        OS_Flags.bTimeout = 0;

        if (bEvent)
        {
            if (_OS_CurTask->State.bReady)
            {
                #ifdef OS_ENABLE_TTIMERS
                    OS_TASK_ATOMIC_WRITE_A(_OS_CurTask->State.bDelay = 0);
                #endif

                return 1;

            }

            _OS_bTaskReady = 1;

        } else {
            _OS_bTaskReady = 0;
        }

        #ifdef OS_ENABLE_TTIMERS

            if (!_OS_CurTask->State.bDelay && _OS_CurTask->State.bCanContinue)
            {
                _OS_bTaskReady = 1;     // *91103*
                OS_Flags.bTimeout = 1;

                return 1;

            }

        #endif

        return 0;

    }

//------------------------------------------------------------------------------
#endif  // !defined(_OS_CheckEvent_DEFINED)
//------------------------------------------------------------------------------


