/*
 ***********************************************************************************************
 *
 *  OSA cooperative RTOS for microcontrollers PIC (Microchip) and 8-bit AVR (Atmel)
 *
 *  OSA is distributed under BSD license (see license.txt)
 *
 *  URL:        http://wiki.pic24.ru/doku.php/en/osa/ref/intro
 *              http://picosa.narod.ru
 *
 *----------------------------------------------------------------------------------------------
 *
 *  File:       osa_tasks.h
 *
 *  Programmer: Timofeev Victor
 *              osa@pic24.ru, testerplus@mail.ru
 *
 *  Definition: Services for work with tasks
 *
 *  History:    26.02.2009   Bug fixed: OS_Task_Replace did not reserve created task by fncall
 *                           OS_ReserveTask renamed to OS_Task_Reserve
 *              01.04.2009   All processor specific services for tasks (create, replace and
 *                           reserve) were moved into proc specific h-file in "port" folder
 *              01.07.2009 - Removed brasses around parametr pTask in all macros (because CCS
 *                           compiler exited with error on expression (pTask)->...
 *              03.11.2009 - Service OS_Task_Replace_P added
 * 
 *              18.12.2009 - OS_Task_Pause and OS_Task_Delete were corrected: these services
 *                           made a comparison of _OS_CurTask's and service argument's addresses,
 *                           instead of comparing their values
 * 
 *              19.12.2009 - _OS_Return replaced with OS_Yield in service OS_Task_Pause (the
 *                           return address did not stored on exit task)
 * 
 *              11.03.2010 -    4.  OS_Task_Create_M service added
 *                              11. OS_Task_GetCur() service added (same as OS_GetCurTask())
 *                              13. OS_Task_GetCreated() service added
 *
 ***********************************************************************************************
 */



/************************************************************************************************
 *                                                                                              *
 *                                   T A S K S                                                  *
 *                                                                                              *
 ************************************************************************************************/


//------------------------------------------------------------------------------

#if defined(OS_PROTECT_MEMORY_ACCESS)
    #define __OS_TASK_DI_A()  OS_DI_INT()
    #define __OS_TASK_RI_A()  OS_RI_INT()
#else
    #define __OS_TASK_DI_A()
    #define __OS_TASK_RI_A()
#endif


#define OS_TASK_ATOMIC_WRITE_A(expr)    \
    OSM_BEGIN {                         \
        __OS_TASK_DI_A();               \
        expr;                           \
        __OS_TASK_RI_A();               \
    } OSM_END

#define OS_TASK_ATOMIC_READ_A(expr)                                         \
    (__OS_TASK_DI_A(), _OS_temp = (expr), __OS_TASK_RI_A(), _OS_temp)       \

//------------------------------------------------------------------------------



#define this_task   _OS_CurTask

/************************************************************************/
/*                                                                      */
/* Stop current task                                                    */
/*                                                                      */
/************************************************************************/

#define OS_Task_Stop()   OS_Task_Delete(this_task)


/************************************************************************/
/*                                                                      */
/* Delete task                                                          */
/*                                                                      */
/************************************************************************/

#define OS_Task_Delete(pTask)                                           \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        OS_TASK_ATOMIC_WRITE_A(pTask->State.bEnable = 0);               \
        if ((pTask) == (_OS_CurTask))       /*91218*/                   \
        {                                                               \
            _OS_Return();                                               \
        }                                                               \
        _OS_CLR_IRP();          /*90922*/                               \
    }


/************************************************************************/
/*                                                                      */
/* Pause task                                                           */
/*                                                                      */
/************************************************************************/

#define OS_Task_Pause(pTask)                                            \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        OS_TASK_ATOMIC_WRITE_A(pTask->State.bPaused = 1);               \
        if ((pTask) == (_OS_CurTask))        /*91218*/                  \
        {                                                               \
            OS_Yield();                      /*91219*/                  \
        }                                                               \
        _OS_CLR_IRP();          /*90922*/                               \
    }

/************************************************************************/
/*                                                                      */
/* Continue paused task                                                 */
/*                                                                      */
/************************************************************************/

#define OS_Task_Continue(pTask)                                         \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        OS_TASK_ATOMIC_WRITE_A(pTask->State.bPaused = 0);               \
        _OS_CLR_IRP();          /*90922*/                               \
    }


/************************************************************************/
/*                                                                      */
/* Replace current task with new (current task will be deleted)         */
/*                                                                      */
/************************************************************************/

// See proc.spec. h-file in "port" folder.



/****************************************************************************/
/*                                                                          */
/* Create task                                                              */
/*                                                                          */
/****************************************************************************/

// See proc.spec. h-file in "port" folder.



/************************************************************************/
/*                                                                      */
/* Create task by indirrect address                                     */
/*                                                                      */
/************************************************************************/

#define OS_Task_CreateP(priority, TaskAddr)      _OS_Task_Create(priority, TaskAddr)
#define OS_Task_Create_P(priority, TaskAddr)     _OS_Task_Create(priority, TaskAddr)

// If task created indirrectly then it is nessesary to insert in
// the main() service OS_ReserveTasks:
// see proc.spec. h-file in "port" folder.



/************************************************************************/
/*                                                                      */
/* Get/set current task priority                                        */
/*                                                                      */
/************************************************************************/

#ifdef __OSAMIKROC16__
#define OS_Task_GetPriority(pTask)      _OS_Task_GetPriority(pTask)
#else
#define OS_Task_GetPriority(pTask)      pTask->State.Priority
#endif

#define OS_Task_SetPriority(pTask, new_priority)                        \
    {                                                                   \
        _OS_SET_IRP_CUR_TASK(); /*90922*/                               \
        pTask->State.Priority = new_priority;                           \
        _OS_CLR_IRP();          /*90922*/                               \
    }



/************************************************************************/
/*                                                                      */
/* Get current task's descriptor address                                */
/*                                                                      */
/************************************************************************/

#define OS_GetCurTask()                 this_task
#define OS_Task_GetCur()                this_task       /*100311.11*/


/************************************************************************/
/*                                                                      */
/* Check for task is paused                                             */
/*                                                                      */
/************************************************************************/

#ifdef __OSAMIKROC16__
#define OS_Task_IsPaused(pTask)         _OS_Task_IsPaused(pTask)
#else
#define OS_Task_IsPaused(pTask)         pTask->State.bPaused
#endif





/************************************************************************************************
 *                                                                                              *
 *                                    T A S K   T I M E R S                                     * 
 *                                                                                              *
 ************************************************************************************************/


/************************************************************************/
/*                                                                      */
/* Delay current task                                                   */
/*                                                                      */
/************************************************************************/
#define OS_Delay(delaytime)                        \
    {                                              \
        _OS_InitDelay(delaytime);                  \
        OS_Yield();                                \
    }                                              \

/*100210.3*/ 
#define OS_Ttimer_Delay(delaytime)   OS_Delay(delaytimer)

/************************************************************************/
// Run task timer for user purpose
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_Run(delaytime)                    \
    {                                               \
        _OS_InitDelay(delaytime);                   \
        _OS_SetCanContinue();                       \
    }                                               \

/************************************************************************/
// Check task timer for timeout
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_Check()            !_OS_CurTask->State.bDelay

/************************************************************************/
// Check task timer is run
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_IsRun()            _OS_CurTask->State.bDelay


/************************************************************************/
// Get task timer's value
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_Get()              (-_OS_CurTask->Timer)

/************************************************************************/
// Wait for task timer will overflow
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_Wait()             OS_Wait(OS_Ttimer_Check())


/************************************************************************/
// Break timer and clear counter
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_Break()                           \
    OS_TASK_ATOMIC_WRITE_A(                         \
        _OS_CurTask->State.bDelay = 0;              \
        _OS_CurTask->Timer        = 0;              \
    )

/************************************************************************/
// Pause timer
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_Pause()           OS_TASK_ATOMIC_WRITE_A(_OS_CurTask->State.bDelay = 0)

/************************************************************************/
// Continue paused timer
/************************************************************************/
/*100210.3*/ 
#define OS_Ttimer_Continue()        OS_TASK_ATOMIC_WRITE_A(_OS_CurTask->State.bDelay = 1)

/************************************************************************/
// Create task inside main() (don't requires OS_Task_Reserve usage)
/************************************************************************/
/*100311.4*/ 
#ifndef OS_Task_Create_M
#define OS_Task_Create_M(p, t)      OS_Task_Create(p, t)
#endif


#define OS_Task_GetCreated()        &OS_TaskVars[_OS_temp]  /*100311.13*/
